--*******************************************************************************************************************************
--
-- Description: Model 376 top level entity.
--				Note: This entity is common to all SigFPGA products, but tailored to the Model 376.
--					
-- Rev History: 09-04-2013:	Initial release.
--                                                  
--*******************************************************************************************************************************
                                             
library ieee,unisim;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use unisim.VCOMPONENTS.all;
use work.model_specific_package.all;
use work.sigfpga_package.all;

entity SigFPGA_Top is
port (
	-- PCI Express bus signals produced by the Xilinx Endpoint Block Plus for PCI Express core generator.
	sys_reset_n		: in std_logic;						-- Refer to the Xilinx PCI Express documentation.
	
	pci_sys_clk_p	: in std_logic;						-- Refer to the Xilinx PCI Express documentation.
	pci_sys_clk_n	: in std_logic;						-- Differential pair (see above).
	
	pci_exp_rxp		: in std_logic_vector(PCI_LANES-1 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	pci_exp_rxn		: in std_logic_vector(PCI_LANES-1 downto 0);	-- Differential pair (see above).
	
	pci_exp_txp		: out std_logic_vector(PCI_LANES-1 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	pci_exp_txn		: out std_logic_vector(PCI_LANES-1 downto 0);	-- Differential pair (see above).
	
	-- 200 MHz clock from an external oscillator.
	clk200_p		: in std_logic;						-- 200 MHz clock for IDELAY referenc.
	clk200_n		: in std_logic;						-- Differential pair (see above).
	
	-- RX data, over-range (ovr) and forwarded clock signals.
	rx1_data_in_p	: in std_logic_vector((RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- RX channel #1 data bus.
	rx1_data_in_n	: in std_logic_vector((RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- Differential pair (see above).
	   
	rx1_ovr_in_p	: in std_logic;						-- RX channel #1 over-range indicator.
	rx1_ovr_in_n	: in std_logic;						-- Differential pair (see above).
	                                                                                                   
	rx1_clk_in_p	: in std_logic;						-- RX channel #1 forwarded clock.
	rx1_clk_in_n	: in std_logic;						-- Differential pair (see above).
	                
	rx2_data_in_p	: in std_logic_vector((RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- RX channel #2 data bus.
	rx2_data_in_n	: in std_logic_vector((RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- Differential pair (see above).
	
	rx2_ovr_in_p	: in std_logic;						-- RX channel #2 over-range indicator.
	rx2_ovr_in_n	: in std_logic;						-- Differential pair (see above).
	
	rx2_clk_in_p	: in std_logic;						-- RX channel #2 forwarded clock.
	rx2_clk_in_n	: in std_logic;						-- Differential pair (see above).
	
	-- QDR II+ SRAM clock from an external oscillator.
	qdriip_sys_clk_p		: in  std_logic;						-- QDR II+ SRAM system clock for both SRAM-A and SRAM-B.
	qdriip_sys_clk_n		: in  std_logic;						-- Differential pair (see above).
	
	-- QDR II+ SRAM-A interface signals.
	sram_a_qdriip_cq		: in  std_logic;						-- QDR II+ SRAM echo clock.
	sram_a_qdriip_q			: in  std_logic_vector(17 downto 0);	-- QDR II+ SRAM synchronous data outputs.
	sram_a_qdriip_qvld		: in  std_logic;						-- QDR II+ SRAM read data valid flag (not used).
	sram_a_qdriip_k_p		: out std_logic;						-- QDR II+ SRAM input clock.
	sram_a_qdriip_k_n		: out std_logic;						-- QDR II+ SRAM differential pair (see above).
	sram_a_qdriip_d			: out std_logic_vector(17 downto 0);	-- QDR II+ SRAM synchronous data inputs.
	sram_a_qdriip_sa		: out std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- QDR II+ SRAM synchronous address inputs.
	sram_a_qdriip_w_n		: out std_logic;						-- QDR II+ SRAM synchronous active low write enable.
	sram_a_qdriip_r_n		: out std_logic;						-- QDR II+ SRAM synchronous active low read enable.
	sram_a_qdriip_bw_n		: out std_logic_vector(1 downto 0);		-- QDR II+ SRAM synchronous byte enables.
	sram_a_qdriip_dll_off_n	: out std_logic;						-- QDR II+ SRAM active low DLL disable.
	
	-- QDR II+ SRAM-B interface signals.
	sram_b_qdriip_cq		: in  std_logic;						-- QDR II+ SRAM echo clock.
	sram_b_qdriip_q			: in  std_logic_vector(17 downto 0);	-- QDR II+ SRAM synchronous data outputs.
	sram_b_qdriip_qvld		: in  std_logic;						-- QDR II+ SRAM read data valid flag (not used).
	sram_b_qdriip_k_p		: out std_logic;						-- QDR II+ SRAM input clock.
	sram_b_qdriip_k_n		: out std_logic;						-- QDR II+ SRAM differential pair (see above).
	sram_b_qdriip_d			: out std_logic_vector(17 downto 0);	-- QDR II+ SRAM synchronous data inputs.
	sram_b_qdriip_sa		: out std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Synchronous address inputs to the SRAM.
	sram_b_qdriip_w_n		: out std_logic;						-- QDR II+ SRAM synchronous active low write enable.
	sram_b_qdriip_r_n		: out std_logic;						-- QDR II+ SRAM synchronous active low read enable.
	sram_b_qdriip_bw_n		: out std_logic_vector(1 downto 0);		-- QDR II+ SRAM synchronous byte enables.
	sram_b_qdriip_dll_off_n	: out std_logic;						-- QDR II+ SRAM active low DLL disable..
	
	-- Inter-integrated circuit (I2C) bus signals.
	i2c_scl			: out std_logic;					-- I2C programming clock.
	i2c_sda			: inout std_logic;					-- I2C bidirectional data to peripheral chips.
	
	-- Serial peripheral interface (SPI) bus signals.
	spi_clk			: out std_logic;					-- SPI programming clock.
	spi_sdi			: out std_logic;					-- SPI data output to peripheral chips.
	spi_sdo			: in std_logic;						-- SPI data input from peripheral chips.
	spi_csb_cmd     : out std_logic_vector(2 downto 0); -- SPI chip select command interface (Bit 0: Clock, Bit 1: Data, Bit 2: Enable).
	
	-- Frequency synthesizer command interface signals, applies only to the programmable synthesizer option.
	synth_cmd		: out std_logic_vector(2 downto 0);	-- Dedicated serial command bus to the frequency synthesizer (Bit 0: Clock, Bit 1: Data, Bit 2: Enable). 
	
	-- LED control signals.
	led_on			: out std_logic_vector(1 downto 0);	-- Active high control to switch on LEDs.
	
	-- Hardware trigger inputs.
	coax_trig		: in  std_logic;					-- Trigger input on coaxial cable.
	gpio_trig		: in  std_logic;					-- Trigger input on GPIO cable.
	
	-- Uncommitted bidirectional IO pins (GPIO connector and user connector).
	gpio			: inout std_logic_vector(GPIO_COUNT downto 1);	-- GPIO data pins.
	gpio_tristate	: out std_logic;								-- Tristate all GPIO voltage translators.
	user			: inout std_logic_vector(USER_COUNT downto 1);	-- User data pins.
	user_tristate	: out std_logic;								-- Tristate all user voltage translators.
	gpio_user_cmd	: out std_logic_vector(2 downto 0) 				-- GPIO/user voltage translator command interface (Bit 0: Clock, Bit 1: Data, Bit 2: Load).
);                                                                                            
end SigFPGA_Top;

architecture structural of SigFPGA_Top is

	signal sys_rst_n		: std_logic;					-- Internal system reset.
	signal pci_sys_clk			: std_logic;					-- Internal single ended version of PCI Express reference clock.
	signal clk200_iob		: std_logic;					-- Internal single ended version of differential 200 MHz clock.
	signal sample_clk_iob	: std_logic;					-- Internal single ended version of differential sample clock. 
	signal rx_data_in_iob	: RX_BUS_IN;					-- Internal RX data bus.
	signal rx_ovr_in_iob	: RX_1;							-- Internal RX over-range indicator.
	signal rx_clk_in_iob	: RX_1;							-- Internal RX forwarded clock.
	signal qdriip_cq		: DUAL_1;						-- Echo clock.
	signal qdriip_q			: DUAL_18;						-- Synchronous data outputs from the SRAM.
	signal qdriip_qvld		: DUAL_1;						-- Echo clock.
	signal qdriip_k_p		: DUAL_1;						-- Input clock.
	signal qdriip_k_n		: DUAL_1;						-- Differential pair (see above).
	signal qdriip_d			: DUAL_18;						-- Synchronous data inputs to the SRAM.
	signal qdriip_sa		: DUAL_QDR;						-- Synchronous address inputs to the SRAM.
	signal qdriip_w_n		: DUAL_1;						-- Synchronous active low write enable.
	signal qdriip_r_n		: DUAL_1;						-- Synchronous active low read enable.
	signal qdriip_bw_n		: DUAL_2;						-- Synchronous byte enable inputs to the SRAM.
	signal qdriip_dll_off_n	: DUAL_1;						-- Active low SRAM DLL disable.
	signal led_on_iob		: std_logic_vector(1 downto 0);	-- Internal active high control to switch on LEDs.
    signal i2c_scl_iob		: std_logic;					-- Internal I2C programming clock.
    signal i2c_sda_in		: std_logic;					-- Internal I2C data input from peripheral chips.
    signal i2c_sda_out		: std_logic;					-- Internal I2C data output to peripheral chips.
    signal i2c_direction	: std_logic;					-- Select I2C direction as input (1) or output (0).
    signal spi_clk_iob		: std_logic;					-- Internal SPI programming clock.
    signal spi_sdi_iob		: std_logic;					-- Internal SPI data output to peripheral chips.
    signal spi_sdo_iob		: std_logic;					-- Internal SPI data input from peripheral chips.
    signal spi_csb_cmd_iob	: std_logic_vector(2 downto 0);	-- Internal SPI chip select command interface.
    signal synth_cmd_iob	: std_logic_vector(2 downto 0);	-- Internal frequency synthesizer command interface. 
    signal coax_trig_iob	: std_logic;					-- Trigger input on coaxial cable.
    signal gpio_trig_iob	: std_logic;					-- Trigger input on GPIO cable.
    signal gpio_in			: std_logic_vector(GPIO_COUNT downto 1);	-- Internal GPIO inputs. 
    signal gpio_out			: std_logic_vector(GPIO_COUNT downto 1);	-- Internal GPIO outputs. 
    signal gpio_direction	: std_logic_vector(GPIO_COUNT downto 1);	-- Set the direction of GPIO pins to outputs (0) or inputs (1). 
    signal gpio_tristate_iob	: std_logic;					-- Tristate all GPIO voltage translators. 
    signal user_in			: std_logic_vector(USER_COUNT downto 1);	-- Internal user bus inputs. 
    signal user_out			: std_logic_vector(USER_COUNT downto 1);	-- Internal user bus outputs. 
    signal user_direction	: std_logic_vector(USER_COUNT downto 1);	-- Direction of user pin. 
    signal user_tristate_iob	: std_logic;					-- Tristate all user voltage translators. 
    signal gpio_user_cmd_iob: std_logic_vector(2 downto 0);	-- Internal GPIO/user voltage translator command interface.

begin

    XCVR : entity work.transceiver
	port map (
		sys_rst_n		=> sys_rst_n,
		pci_sys_clk			=> pci_sys_clk,
		pci_exp_rxp		=> pci_exp_rxp,
		pci_exp_rxn		=> pci_exp_rxn,
		pci_exp_txp		=> pci_exp_txp,
		pci_exp_txn		=> pci_exp_txn,
		clk200_in		=> clk200_iob,
		sample_clk_in	=> sample_clk_iob,
		rx_data_in 		=> rx_data_in_iob,
		rx_ovr_in 		=> rx_ovr_in_iob,
		rx_clk_in 		=> rx_clk_in_iob,
		tx_data_out 	=> open,
		tx_clk_in 		=> "0",
		tx_clk_out		=> open,
		qdriip_sys_clk_p	=> qdriip_sys_clk_p,
		qdriip_sys_clk_n	=> qdriip_sys_clk_n,
		qdriip_cq		=> qdriip_cq,
		qdriip_q		=> qdriip_q,
		qdriip_qvld		=> qdriip_qvld,
		qdriip_k_p		=> qdriip_k_p,
		qdriip_k_n		=> qdriip_k_n,
		qdriip_d		=> qdriip_d,
		qdriip_sa		=> qdriip_sa,
		qdriip_w_n		=> qdriip_w_n,
		qdriip_r_n		=> qdriip_r_n,
		qdriip_bw_n		=> qdriip_bw_n,
		qdriip_dll_off_n=> qdriip_dll_off_n,
		i2c_scl 		=> i2c_scl_iob,
		i2c_sda_in 		=> i2c_sda_in,
		i2c_sda_out 	=> i2c_sda_out,
		i2c_direction 	=> i2c_direction,
		spi_clk 		=> spi_clk_iob,
		spi_sdi 		=> spi_sdi_iob,
		spi_sdo 		=> spi_sdo_iob,
		spi_csb_cmd 	=> spi_csb_cmd_iob,
		synth_cmd		=> synth_cmd_iob,
		led_on 			=> led_on_iob,
		coax_trig		=> coax_trig_iob,
		gpio_trig		=> gpio_trig_iob,
		gpio_in			=> gpio_in,
		gpio_out		=> gpio_out,
		gpio_direction	=> gpio_direction,
		gpio_tristate	=> gpio_tristate_iob,
		user_in			=> user_in,
		user_out		=> user_out,
		user_direction	=> user_direction,
		user_tristate	=> user_tristate_iob,
		gpio_user_cmd	=> gpio_user_cmd_iob
	);

    -- Assign the PCI Express differential system clock to a global clock tree.                    
    REFCLK_IBUF : IBUFDS_GTE2 port map (I => pci_sys_clk_p, IB => pci_sys_clk_n, O => pci_sys_clk, CEB => '0', ODIV2 => open);

	--	Map the differential 200 MHz external clock input to a global clock buffer.	
	CLK200_IBUF : IBUFGDS
		generic map (DIFF_TERM  => FALSE, IOSTANDARD => "LVDS_25")
		port map (I => clk200_p, IB => clk200_n, O => clk200_iob);

    --	The RX sample clock is too fast for the FPGA, use the divided RX1 data clock instead.
    sample_clk_iob <= rx_clk_in_iob(1);

	-- Map the system reset to an internal signal.
	SYS_RESET_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => sys_rst_n, I => sys_reset_n);
		
	-- Map the LVDS RX data pin pairs to an internal data bus.
	RX_DATA_BUF : for i in 0 to (RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 generate
		RX1_DATA_IBUF : IBUFDS  -- Channel 1
			generic map (DIFF_TERM  => TRUE, IOSTANDARD => "LVDS_25")
			port map (I => rx1_data_in_p(i), IB => rx1_data_in_n(i), O => rx_data_in_iob(1)(i));
		RX2_DATA_IBUF : IBUFDS  -- Channel 2
			generic map (DIFF_TERM  => TRUE, IOSTANDARD => "LVDS_25")
			port map (I => rx2_data_in_p(i), IB => rx2_data_in_n(i), O => rx_data_in_iob(2)(i));
	end generate;
	
	--	Map the LVDS RX over-range pin pairs to internal signals.
	RX1_OVR_IBUF : IBUFDS  -- Channel 1
		generic map (IOSTANDARD => "LVDS_25")
		port map (I => rx1_ovr_in_p, IB => rx1_ovr_in_n, O => rx_ovr_in_iob(1));
	RX2_OVR_IBUF : IBUFDS  -- Channel 2
		generic map (IOSTANDARD => "LVDS_25")
		port map (I => rx2_ovr_in_p, IB => rx2_ovr_in_n, O => rx_ovr_in_iob(2));
	
	--	Map the LVDS RX forwarded clock pin pairs to internal data ready signals.
	RX1_CLK_IBUF : IBUFDS  -- Channel 1
		generic map (IOSTANDARD => "LVDS_25")
	 	port map (I  => rx1_clk_in_p, IB => rx1_clk_in_n, O => rx_clk_in_iob(1));
	RX2_CLK_IBUF : IBUFDS  -- Channel 2
		generic map (IOSTANDARD => "LVDS_25")
	 	port map (I  => rx2_clk_in_p, IB => rx2_clk_in_n, O => rx_clk_in_iob(2));
	 	
	-- Assign the I2C bus pins to I/O buffers.
	I2C_SCL_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => i2c_scl_iob, O => i2c_scl);
	I2C_SDA_OBUF : IOBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => i2c_sda_in, IO => i2c_sda, I => i2c_sda_out, T => i2c_direction);
		
	-- Assign the SPI bus pins to I/O buffers.
	SPI_CLK_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => spi_clk_iob, O => spi_clk);
	SPI_SDI_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => spi_sdi_iob, O => spi_sdi);
	SPI_SDO_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => spi_sdo_iob, I => spi_sdo);
	SPI_CSB_CMD_BUF : for i in 0 to 2 generate
		SPI_CSB_CMD_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => spi_csb_cmd_iob(i), O => spi_csb_cmd(i));
	end generate;
		
	-- Assign the frequency synthesizer command bus pins to I/O buffers.
	SYNTH_BUF : for i in 0 to 2 generate
		SYNTH_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => synth_cmd_iob(i), O => synth_cmd(i));
	end generate;
		
	--	Assign the LED controls to output buffers.
	LED_BUF : for i in 0 to 1 generate
		LED_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => led_on_iob(i), O => led_on(i));
	end generate;
	 	
	-- Map the trigger pins to internal signals.
	COAX_TRIG_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => coax_trig_iob, I => coax_trig);
	GPIO_TRIG_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => gpio_trig_iob, I => gpio_trig);
	
	-- Map the GPIO pins to an internal bus.
	GPIO_BUF : for i in 1 to GPIO_COUNT generate
		GPIO_IOBUF : IOBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (O => gpio_in(i), IO => gpio(i), I => gpio_out(i), T => gpio_direction(i));
	end generate;
	GPIO_TRISTATE_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => gpio_tristate_iob, O => gpio_tristate);
		
	-- Map the user pins to an internal bus.
	USER_GT50 : if USER_COUNT > 50 generate
		DIRECT1 : for i in 1 to 50 generate
			DIRECT1_IOBUF : IOBUF
				generic map (IOSTANDARD => "LVCMOS25")
				port map (O => user_in(i), IO => user(i), I => user_out(i), T => user_direction(i));
		end generate;
		XLATOR : for i in 51 to USER_COUNT generate
			XLATOR_IOBUF : IOBUF
				generic map (IOSTANDARD => "LVCMOS18")
				port map (O => user_in(i), IO => user(i), I => user_out(i), T => user_direction(i));
		end generate;
	end generate;
	USER_LT50 : if USER_COUNT <= 50 generate
		DIRECT2 : for i in 1 to USER_COUNT generate
			DIRECT2_IOBUF : IOBUF
				generic map (IOSTANDARD => "LVCMOS25")
				port map (O => user_in(i), IO => user(i), I => user_out(i), T => user_direction(i));
		end generate;
	end generate;
	USER_TRISTATE_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => user_tristate_iob, O => user_tristate);

	-- Map the GPIO/user voltage translator command interface pins to an internal bus.
	GPIO_USER_CMD_BUF : for i in 0 to 2 generate
		GPIO_USER_CMD_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => gpio_user_cmd_iob(i), O => gpio_user_cmd(i));
	end generate;

	-- Convert all of the signals from the two QDR SRAM interface signals into a single indexed signal name.
	-- Note: I/O buffer assignments are performed by the IP core.
	qdriip_cq(0) <= sram_a_qdriip_cq;
	qdriip_q(0) <= sram_a_qdriip_q;
	qdriip_qvld(0) <= sram_a_qdriip_qvld;
	sram_a_qdriip_k_p <= qdriip_k_p(0);
	sram_a_qdriip_k_n <= qdriip_k_n(0);
	sram_a_qdriip_d <= qdriip_d(0);
	sram_a_qdriip_sa <= qdriip_sa(0);
	sram_a_qdriip_w_n <= qdriip_w_n(0);
	sram_a_qdriip_r_n <= qdriip_r_n(0);
	sram_a_qdriip_bw_n <= qdriip_bw_n(0);
	sram_a_qdriip_dll_off_n <= qdriip_dll_off_n(0);
	
	qdriip_cq(1) <= sram_b_qdriip_cq;
	qdriip_q(1) <= sram_b_qdriip_q;
	qdriip_qvld(1) <= sram_b_qdriip_qvld;
	sram_b_qdriip_k_p <= qdriip_k_p(1);
	sram_b_qdriip_k_n <= qdriip_k_n(1);
	sram_b_qdriip_d <= qdriip_d(1);
	sram_b_qdriip_sa <= qdriip_sa(1);
	sram_b_qdriip_w_n <= qdriip_w_n(1);
	sram_b_qdriip_r_n <= qdriip_r_n(1);
	sram_b_qdriip_bw_n <= qdriip_bw_n(1);
	sram_b_qdriip_dll_off_n <= qdriip_dll_off_n(1);
		
end structural;