--*******************************************************************************************************************************
--
-- Description: This module performs all PCI Express write/read operations to the registers assigned to individual channels.
--				The top four bits of the address are decoded to retrieve the target channel number.
--				The AXI_WIDTH constant determines whether slave_dataout is 64-bit or 128-bit aligned.
--					
-- Rev History: 09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.model_specific_package.all;

entity bar2_channel is

generic(
	CHANNEL_NUMBER : integer
);
	
port (                                                                                                                              
    -- Reset and clock.                                                                                      
	rst    				: in std_logic;									-- Active high asynchronous reset.
	clk		        	: in std_logic;									-- Bus clock.
	
    -- PCI Express write transaction interface.
	slv_addr 			: in std_logic_vector(15 downto 2);				-- Refer to the PLDA EZDMA2 reference manual.
    slv_dataout     	: in std_logic_vector(AXI_WIDTH-1 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	slv_bytevalid 		: in std_logic_vector((AXI_WIDTH/8)-1 downto 0);-- Refer to the PLDA EZDMA2 reference manual.
	bar2_write 			: in std_logic;									-- Logic one indicates that the current PCI write request is directed at BAR2.
                                                                                                                              
    -- PCI Express read transaction interface.
	bar2_slave_read			: in std_logic;								-- Logic one indicates that the current PCI read request is directed at BAR2.
	bar2_rddata_channel	: out std_logic_vector(AXI_WIDTH-1 downto 0);	-- Data return from read operation on BAR2 common register.
    dma_rdaddr      	: in std_logic_vector(15 downto 3);				-- Refer to the PLDA EZDMA2 reference manual.
	                                                                                                                       	       	                                                                                                                       	       
    -- Command and status signals for a channel.
	synchronizer		: out std_logic_vector(35 downto 0);			-- Register containing the synchronizer configuration.
	synchronizer_on_off	: out std_logic_vector(1 downto 0);				-- Active high on/off (bit 0/bit 1) switch for the synchronizer.
	synchronizer_status	: in std_logic_vector(1 downto 0);				-- Indicator bits for busy (0) and ready (1) status reporting.

    -- Channel status and error signals.
	channel_monitor 	: in std_logic_vector(2 downto 0); 				-- Bus used to monitor DMA refresh status and channel errors.
	channel_status 		: out std_logic_vector(2 downto 0);  			-- Bus used to convey channel status since last queried.
	channel_int_mask	: out std_logic_vector(2 downto 0);				-- Mask bits to suppress interrupts based on channel status updates.
	address_error 		: out std_logic_vector(1 downto 0)				-- Logic one indicates an invalid address for this decoder.
	
);
end bar2_channel;

architecture structural of bar2_channel is
                                                                                                                               	
	-- BAR2 receiver channel configration registers.	                                                                                                           	
	signal bytevalid				: std_logic_vector(1 downto 0);		-- Logic one indicates that all PCI byte valids are active.
	signal channel_status_reg		: std_logic_vector(2 downto 0);		-- Bus used to monitor channel status since last cleared.
	signal channel_status_read		: std_logic_vector(2 downto 0);		-- Bus used to store channel status before a clear so it can be read.
	signal channel_int_mask_reg		: std_logic_vector(2 downto 0);		-- Mask bits to suppress interrupts based on channel status updates.
	signal synchronizer_reg			: std_logic_vector(35 downto 0);	-- Register containing start and stop event selections.
	
begin

--***************************************************** WRITE TRANSACTIONS *****************************************************

   process (clk,rst)
    begin
        if rst='1' then
            bytevalid <= (others => '0');
			synchronizer_on_off <= "00";
           	synchronizer_reg <= (others => '0');
           	channel_status_reg <= (others => '0');
           	channel_status_read <= (others => '0');
           	channel_int_mask_reg <= (others => '0');
			address_error(0) <= '0';

        elsif rising_edge(clk) then

        	-- Create an active high enable for write operations to the upper 32 register bits.
        	if slv_addr(3) = '0' then
        		bytevalid(1) <= (slv_bytevalid(7) and slv_bytevalid(6) and slv_bytevalid(5) and slv_bytevalid(4)) or slv_addr(2);
        	else
        		bytevalid(1) <= (slv_bytevalid((AXI_WIDTH/8)-1) and slv_bytevalid((AXI_WIDTH/8)-2) and slv_bytevalid((AXI_WIDTH/8)-3) and slv_bytevalid((AXI_WIDTH/8)-4)) or slv_addr(2);
			end if;        		
        	-- Create an active high enable for write operations to the lower 32 register bits.
        	if slv_addr(3) = '0' then
        		bytevalid(0) <= slv_bytevalid(3) and slv_bytevalid(2) and slv_bytevalid(1) and slv_bytevalid(0) and not(slv_addr(2));
        	else
        		bytevalid(0) <= slv_bytevalid((AXI_WIDTH/8)-5) and slv_bytevalid((AXI_WIDTH/8)-6) and slv_bytevalid((AXI_WIDTH/8)-7) and slv_bytevalid((AXI_WIDTH/8)-8) and not(slv_addr(2));
        	end if;
        
            -- Target write decoder that loads slave data into the selected register for a target write completion.
            if bar2_write = '1' and to_integer(unsigned(slv_addr(15 downto 12))) = CHANNEL_NUMBER then
                case slv_addr(11 downto 3) & "000" is
			
					-- Synchronizer state machine on/off command.
                    when x"000" =>
						if bytevalid(0) = '1' then
							if slv_dataout(0) = '1' then
								-- Activate the "on" switch for one clock period.
								synchronizer_on_off(0) <= '1';
							else
								-- Activate the "off" switch for one clock period.
								synchronizer_on_off(1) <= '1';
							end if;
						end if;
						
					-- x"010" Channel status update command defined below.
                    when x"010" =>
                    	null;
					
					-- Channel status interrupt mask.
                    when x"018" =>
						if bytevalid(0) = '1' then
							channel_int_mask_reg(2 downto 1) <= slv_dataout((AXI_WIDTH-64)+5 downto (AXI_WIDTH-64)+4);
							channel_int_mask_reg(0) <= slv_dataout(AXI_WIDTH-64);
						end if;
						
					-- Synchronizer command register.
                    when x"020" =>
						if bytevalid(1) = '1' then synchronizer_reg(35 downto 4) <= slv_dataout(63 downto 32); end if;
						if bytevalid(0) = '1' then synchronizer_reg(3 downto 0) <= slv_dataout(3 downto 0); end if;
						
					-- Return an error if the address is not valid.
                    when others  =>
						address_error(0) <= '1';
						
                end case;    
			else
				synchronizer_on_off <= "00";
				address_error(0) <= '0';
            end if; 

            -- Decode the write transaction to address x"010" that will simultaneously capture and clear the channel status bits.
			if bar2_write = '1' and slv_addr(15 downto 12) = x"0" and bytevalid(0) = '1' and to_integer(unsigned(slv_addr(15 downto 12))) = CHANNEL_NUMBER and (slv_addr(11 downto 3) & "000") = x"010" then
				-- Capture the channel status so that it can be read by software.
				channel_status_read <= channel_status_reg;
				-- Clear the sticky channel status bits by updating to the instantaneous status.
				channel_status_reg <= channel_monitor;
			else
				-- Update the channel status using sticky bits.
				channel_status_reg <= channel_status_reg or channel_monitor;
			end if;
				
        end if; 
    end process;
    
--***************************************************** READ TRANSACTIONS ******************************************************
    
    process (clk,rst)
    begin
        if rst = '1' then  
            bar2_rddata_channel <= (others => '0');
			address_error(1) <= '0';
			
        elsif rising_edge(clk) then
  
            -- Target read decoder that loads selected register contents onto the DMA bus for a target read completion.
            if bar2_slave_read = '1' and to_integer(unsigned(dma_rdaddr(15 downto 12))) = CHANNEL_NUMBER then
                case dma_rdaddr(11 downto 3) & "000" is
                
					-- Synchronizer state machine status.
					when x"000" =>
						bar2_rddata_channel(63 downto 2) <= (others => '0');
						bar2_rddata_channel(1 downto 0) <= synchronizer_status;
                 
					-- Channel status.
                    when x"010" =>
						bar2_rddata_channel(63 downto 6) <= (others => '0');
						bar2_rddata_channel(5 downto 4) <= channel_status_read(2 downto 1);
						bar2_rddata_channel(3 downto 1) <= (others => '0');
						bar2_rddata_channel(0) <= channel_status_read(0);
                 
					-- Channel status interrupt mask.
                    when x"018" =>
						bar2_rddata_channel((AXI_WIDTH-64)+63 downto (AXI_WIDTH-64)+6) <= (others => '0');
						bar2_rddata_channel((AXI_WIDTH-64)+5 downto (AXI_WIDTH-64)+4) <= channel_int_mask_reg(2 downto 1);
						bar2_rddata_channel((AXI_WIDTH-64)+3 downto (AXI_WIDTH-64)+1) <= (others => '0');
						bar2_rddata_channel(AXI_WIDTH-64) <= channel_int_mask_reg(0);
                 
					-- Synchronizer command register.
                    when x"020" =>
						bar2_rddata_channel(63 downto 32) <= synchronizer_reg(35 downto 4);
						bar2_rddata_channel(31 downto 4) <= (others => '0');
						bar2_rddata_channel(3 downto 0) <= synchronizer_reg(3 downto 0);
                 
					-- Return an error if the address is not valid.
                    when others  =>                                                                                                 
						bar2_rddata_channel(31 downto 0) <= x"badaddb2";
						bar2_rddata_channel <= (others => '0');
						address_error(1) <= '1';                                           
	            end case;
	            
			else
				address_error(1) <= '0';
            end if;
                        
        end if;
    end process;
    
--****************************************************** PORT ASSIGNMENTS ******************************************************

    channel_status <= channel_status_reg;    
    channel_int_mask <= channel_int_mask_reg;    
	synchronizer <= synchronizer_reg;
		
end structural;
