--*******************************************************************************************************************************
--
-- Description: This module rate buffers data for a DMA read transaction from the EZDMA2 core to the channel accepting the data.
--					
-- Rev History: 09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.sigfpga_package.all;

entity dma_read_fifo is
port (
    -- Reset and clocks.
	clear 				: in std_logic;										-- Active high synchronous FIFO clear.
	clk 				: in std_logic;										-- Bus clock.
	tx_clk 				: in std_logic;										-- Transmit clock coincident with the output data.
	
    -- Output data bus and associated command/status signals.
	data_out			: out std_logic_vector(AXI_WIDTH-1 downto 0);		-- Output bus carrying data received from a DMA read transaction.
	enable_64bit		: in std_logic_vector((AXI_WIDTH/64)-1 downto 0);	-- Enables to advance the FIFO when data is read.
	fifo_error			: out std_logic;									-- Logic one indicates a FIFO underflow condition.
	data_valid			: out std_logic;									-- Logic one indicates that valid data is available at the FIFO output.
	
    -- DMA data bus and associated command/status signals.                                                                                      
	dma_enable			: in std_logic;										-- Logic one enables DMA transfers.
	dma_fifo_wr			: in std_logic;										-- Logic one indicates that the PLDA EZDMA2 core is supplying data from a DMA read request.
	dma_wrdata			: in std_logic_vector(AXI_WIDTH-1 downto 0);		-- DMA local data bus used for read transactions.
	dma_burst_size		: in std_logic_vector(13 downto 0);					-- Number of bytes to transfer in each DMA burst.
	dma_go				: out std_logic										-- Initiate a DMA write transaction when enough data is available to satisfy a burst.
);
end dma_read_fifo;

architecture structural of dma_read_fifo is

	signal udf_error	: std_logic_vector((AXI_WIDTH/64)-1 downto 0);		-- FIFO underflow flags.
	signal data_count	: std_logic_vector((11*AXI_WIDTH/64)-1 downto 0);	-- Depth of data currently in the FIFO.
	signal total_space	: AXI_11U;											-- Depth of unused space in the FIFO.
	signal bridge_empty	: std_logic_vector((AXI_WIDTH/64)-1 downto 0);		-- Logic one indicates that the FIFO bridge has no data to transfer.
	signal bridge_full	: std_logic_vector((AXI_WIDTH/64)-1 downto 0);		-- Logic one indicates that the FIFO bridge cannot accept another transfer.
	signal bridge_data	: std_logic_vector(AXI_WIDTH-1 downto 0);			-- Data crossing the FIFO bridge.
	signal bridge_en	: std_logic;										-- Logic one initiates a bridge transfer.	
	signal valid_64bit	: std_logic_vector(AXI_WIDTH-1 downto 0);			-- Logic one indicates that valid data is available at the FIFO output.
	
begin

--**************************************************** FIFO INSTANTIATIONS *****************************************************

    -- Parallel FIFOs (8 kbytes each) are used to interface with the PLDA EZDMA2 core.
	DMA_RD_X2 : for i in 0 to (AXI_WIDTH/64)-1 generate
		RD_FIFO : entity work.fifo_dma_1kx64
		port map (
			clk				=> clk,
			rst				=> clear,
			din				=> dma_wrdata((64*i)+63 downto 64*i),
			wr_en			=> dma_fifo_wr,  
			rd_en			=> bridge_en,
			dout			=> bridge_data((64*i)+63 downto 64*i),
			full			=> open,
			empty			=> bridge_empty(i),
			data_count		=> data_count((11*i)+10 downto 11*i)
		);
	end generate;

	-- Transfer data between FIFOs when possible.
	bridge_en <= '1' when to_integer(unsigned(bridge_empty)) = 0 and to_integer(unsigned(bridge_full)) = 0 else '0';
			
    -- Parallel FIFOs (8 kbytes each) are used to interface with the channel accepting DMA data.
	TX_FIFO_X2 : for i in 0 to (AXI_WIDTH/64)-1 generate
		TX_FIFO : entity work.fifo_channel_1kx64
		port map (
			rst			=> clear,
			wr_clk		=> clk,
			rd_clk		=> tx_clk,
			din			=> bridge_data((64*i)+63 downto 64*i),
			wr_en		=> bridge_en,  
			rd_en		=> enable_64bit(i),
			dout		=> data_out((64*i)+63 downto 64*i),
			full		=> bridge_full(i),
			overflow	=> open,
			empty		=> open,
			valid		=> valid_64bit(i),
			underflow	=> udf_error(i)
		);
	end generate;
	
--***************************************************** PORT ASSIGNMENTS *******************************************************

	-- Initiate a DMA transfer when there is enough data available to satisfy a burst (64-bit AXI).
	TX_GO_X1 : if AXI_WIDTH/64 = 1 generate
		process(clk)
		begin
			if rising_edge(clk) then
				-- Calculate the total space needed to load another burst into the DMA FIFO.
				total_space(0) <= unsigned('0' & data_count(10 downto 0)) + unsigned('0' & dma_burst_size(13 downto 3));
			end if;
		end process;	
		-- Initiate a DMA transfer when there is enough space available to satisfy another burst.
		dma_go <= dma_enable when total_space(0) <= "01000000000" else '0';
	end generate;
	
	-- Initiate a DMA transfer when there is enough data available to satisfy a burst (128-bit AXI).
	TX_GO_X2 : if AXI_WIDTH/64 = 2 generate
		process(clk)
		begin
			if rising_edge(clk) then
				-- Calculate the total space needed to load another burst into the DMA FIFO.
				total_space(0) <= unsigned('0' & data_count(10 downto 0)) + unsigned('0' & dma_burst_size(13 downto 3));
				total_space(AXI_WIDTH/64-1) <= unsigned('0' & data_count(21 downto 11)) + unsigned('0' & dma_burst_size(13 downto 3));
			end if;
		end process;	
		-- Initiate a DMA transfer when there is enough space available to satisfy another burst.
		dma_go <= dma_enable when total_space(0) <= "01000000000" and total_space(AXI_WIDTH/64-1) <= "01000000000" else '0';
	end generate;
	
	-- Report an underflow condition in any FIFO.
	fifo_error <= '1' when to_integer(unsigned(udf_error)) > 0 else '0';
	
	-- Data is valid on all channel FIFOs.
	data_valid <= '1' when to_integer(unsigned(valid_64bit)) > 0 else '0';
		
end structural;
